@@ -36,7 +36,10 @@ class Multiple(argparse.Action):
 def main():
     parser = argparse.ArgumentParser("ecquote")
     parser.add_argument(
-        "-D", "--detailed", action="store_true", help="Print a detailed report"
+        "-D",
+        "--detailed",
+        action="store_true",
+        help="Print a detailed report",
     )
 
     parser.add_argument(
@@ -55,7 +58,9 @@ def main():
     )
 
     parser.add_argument(
-        "--streams", action="store_true", help="List the streams found in the requests"
+        "--streams",
+        action="store_true",
+        help="List the streams found in the requests",
     )
     parser.add_argument("--samples")
     parser.add_argument("--split", action="store_true")
@@ -64,6 +69,8 @@ def main():
     parser.add_argument("-w", "--warnings", action="store_true")
     parser.add_argument("-j", "--json", action="store_true")
     parser.add_argument("--csv", action="store_true")
+    parser.add_argument("--xlsx")
+
     parser.add_argument(
         "--by",
         choices=[
@@ -104,6 +111,11 @@ def main():
     parser.add_argument("--max-charge-core", type=int)
     parser.add_argument("--max-charge-high-frequency", type=int)
 
+    parser.add_argument("-X", "--experimental", action="store_true")
+    parser.add_argument("--commercial", action="store_true")
+    parser.add_argument("--non-commercial", action="store_false", dest="commercial")
+    parser.set_defaults(commercial=None)
+
     parser.add_argument("-r", "--request")
     parser.add_argument("files", metavar="FILES-OR-DIRECTORIES", type=str, nargs="*")
     ARGS = parser.parse_args()
@@ -138,6 +150,9 @@ def main():
     if ARGS.modify:
         config("modify", ARGS.modify)
 
+    if ARGS.experimental:
+        config("experimental", ARGS.experimental)
+
     if ARGS.config:
         for c in ARGS.config:
             k, v = c.split("=")
@@ -154,7 +169,7 @@ def main():
     if ARGS.strict_mode:
         config("strict-mode", True)
 
-    if ARGS.csv or ("destinations" in ARGS.show):
+    if ARGS.csv or ARGS.xlsx or ("destinations" in ARGS.show):
         config("destinations", True)
 
     if ARGS.max_charge_core is not None:
@@ -163,6 +178,9 @@ def main():
     if ARGS.max_charge_high_frequency is not None:
         config("max-charge-high-frequency", ARGS.max_charge_high_frequency)
 
+    if ARGS.commercial is not None:
+        config("commercial", ARGS.commercial)
+
     if ARGS.rest:
         from ecquote.rest import run
 
@@ -247,6 +265,11 @@ def main():
         cart.csv(**extra)
         return
 
+    if ARGS.xlsx:
+        extra = {f"by_{ARGS.by}": True}
+        cart.xlsx(path=ARGS.xlsx, **extra)
+        return
+
     extra = {f"show_{k}": True for k in ARGS.show if k != "all"}
     cart.summary(detailed=ARGS.detailed, **extra)
 
