@@ -15,7 +15,7 @@ from collections import defaultdict
 
 from . import __version__
 from .resources import config, resource
-from .utils import bytes, capture_warnings, log_warning_once, roman
+from .utils import bytes, capture_warnings, log_warning_once, progress, roman
 
 LOG = logging.getLogger(__name__)
 
@@ -35,9 +35,31 @@ def band(volume):
 
     for gb, price in sorted(volume_bands.items()):
         if daily_gb <= gb:
-            return gb, price["euros"], None
+            return gb, price["euros"], None, daily_gb
 
-    return None, None, f"Daily volume exceeds {max(volume_bands.keys())}"
+    return None, None, f"Daily volume exceeds {max(volume_bands.keys())}", daily_gb
+
+
+def data_values_band(data_values):
+
+    daily_data_values_millions = data_values / 365 / 1000 / 1000
+    data_values_band = resource("data-values-bands")
+
+    for million, price in sorted(data_values_band.items()):
+        if daily_data_values_millions <= million:
+            return million, price["euros"], None, daily_data_values_millions
+
+    return (
+        None,
+        None,
+        f"Daily data values exceeds {max(data_values_band.keys())}",
+        daily_data_values_millions,
+    )
+
+
+def commercial(s):
+    s = s.lower()
+    return ("commercial" in s) and ("non-commercial" not in s)
 
 
 class Roman:
@@ -63,47 +85,77 @@ class Roman:
 
 class Coster:
     def __init__(self):
-        self.volume = 0
-        self.fields = 0
+        self.yearly_volume = 0
+        self.yearly_fields = 0
+        self.yearly_data_values = 0
         self.daily_volume = 0
         self.daily_fields = 0
+        self.daily_data_values = 0
         self.requests = []
         self.subcosts = {}
+        self.experimental = config("experimental")
 
     def add(self, *requests):
-        for r in requests:
+        for r in progress(requests, "Volumes"):
             with capture_warnings(r):
-                self.volume += r.estimated_volume() * r.frequency()
-                self.fields += r.number_of_fields() * r.frequency()
+                self.yearly_volume += r.estimated_volume() * r.frequency()
+                self.yearly_fields += r.number_of_fields() * r.frequency()
                 self.daily_volume += r.estimated_volume()
                 self.daily_fields += r.number_of_fields()
+                if self.experimental:
+                    self.yearly_data_values += r.data_values() * r.frequency()
+                    self.daily_data_values += r.data_values()
         self.requests += requests
 
     def as_dict(self):
         result = {
-            "yearly_volume": self.volume,
-            "yearly_fields": self.fields,
+            "yearly_volume": self.yearly_volume,
+            "yearly_fields": self.yearly_fields,
             "worst_daily_volume": self.daily_volume,
             "worst_daily_fields": self.daily_fields,
-            "average_daily_volume": int(self.volume / 365 + 0.5),
-            "average_daily_fields": int(self.fields / 365 + 0.5),
+            "average_daily_volume": int(self.yearly_volume / 365 + 0.5),
+            "average_daily_fields": int(self.yearly_fields / 365 + 0.5),
         }
 
-        band, euro, error = self.band()
+        band, euro, error, ref = self.band()
         if error:
             result["volume_band"] = error
         else:
             result["volume_band"] = band
             result["volume_cost"] = euro
+
+        if self.experimental:
+
+            result["yearly_data_values"] = self.yearly_data_values
+            result["average_daily_data_values"] = int(
+                self.yearly_data_values / 365 + 0.5
+            )
+
+            band, euro, error, ref = self.data_values_band()
+
+            if error:
+                result["data_values_band"] = error
+            else:
+                result["data_values_band"] = band
+                result["data_values_cost"] = euro
+
+            try:
+                result["diff"] = result["data_values_cost"] - result["volume_cost"]
+            except KeyError:
+                result["diff"] = None
+
+            result["data_values_ref"] = ref
+            result["data_values_ratio"] = self.yearly_volume / self.yearly_data_values
+
         return result
 
     def summary(self, title):
         print(title)
         print()
-        print("   Yearly volume:   ", bytes(self.volume))
-        print("   Yearly fields:   ", "{:,}".format(self.fields))
+        print("   Yearly volume:   ", bytes(self.yearly_volume))
+        print("   Yearly fields:   ", "{:,}".format(self.yearly_fields))
 
-        band, euro, error = self.band()
+        band, euro, error, _ = self.band()
         if error:
             print("   Volume band:     ", error)
         else:
@@ -111,7 +163,10 @@ class Coster:
             print("   Volume cost:     ", "EUR {:,}".format(euro))
 
     def band(self):
-        return band(self.volume)
+        return band(self.yearly_volume)
+
+    def data_values_band(self):
+        return data_values_band(self.yearly_data_values)
 
 
 class EPUBased(Coster):
@@ -123,11 +178,9 @@ class EPUBased(Coster):
 
     def add(self, *requests):
         super().add(*requests)
-        for r in requests:
+        for r in progress(requests, "Costing"):
             D, E, M = self.factors(r)
             self.epus += self.compute_epus(r, D, E, M)
-
-        for r in requests:
             self.items += r.number_of_chargeable_items()
 
     def factors(self, request):
@@ -297,7 +350,7 @@ class Costing:
         self.per_category = defaultdict(coster_class)
         self.per_user = defaultdict(coster_class)
         self.per_destination = defaultdict(coster_class)
-        for r in requests:
+        for r in progress(requests, "Sub-totals"):
             self.per_target[r.tag].add(r)
             self.per_group[r.group].add(r)
             self.per_category[r.category].add(r)
@@ -539,9 +592,8 @@ class Costing:
 
         self.totals.summary("Grand total:", self.max_charge_limit)
 
-    def csv(
+    def _rows(
         self,
-        file=sys.stdout,
         by_groups=False,
         by_targets=False,
         by_categories=False,
@@ -550,9 +602,9 @@ class Costing:
     ):
         import csv
 
-        fieldnames = []
         rows = []
         ok = True
+        filter_commercial = config("commercial")
 
         def _(collection, per_collection, categories=None, default={}):
             for name, coster in sorted(per_collection.items()):
@@ -567,6 +619,11 @@ class Costing:
                     if coster.euros >= self.max_charge_limit
                     else int(coster.euros)
                 )
+
+                if filter_commercial is not None:
+                    if r.get("commercial") is not filter_commercial:
+                        continue
+
                 rows.append(r)
 
         if by_categories and ok:
@@ -588,13 +645,16 @@ class Costing:
             default = {}
             path = config("categories")
             if path:
-                default = dict(category="unknown")
+                default = dict(category="unknown", commercial="unknown")
                 categories = defaultdict(set)
                 with open(path) as csvfile:
                     for row in csv.reader(csvfile, delimiter=","):
                         categories[row[2]].add(row[1])
                 for k, v in list(categories.items()):
-                    categories[k] = dict(category=", ".join(sorted(v)))
+                    categories[k] = dict(
+                        category=", ".join(sorted(v)),
+                        commercial=any(commercial(x) for x in v),
+                    )
             _("user", self.per_user, categories, default)
 
         if by_destinations or ok:  # The 'or' is not a typo
@@ -603,19 +663,142 @@ class Costing:
             default = {}
             path = config("categories")
             if path:
-                default = dict(category="unknown", user="unknown")
+                default = dict(category="unknown", user="unknown", commercial="unknown")
                 with open(path) as csvfile:
                     for row in csv.reader(csvfile, delimiter=","):
-                        categories[row[0]] = dict(category=row[1], user=row[2])
+                        categories[row[0]] = dict(
+                            category=row[1],
+                            user=row[2],
+                            commercial=commercial(row[1]),
+                        )
 
             _("destination", self.per_destination, categories, default)
 
+        return rows
+
+    def csv(
+        self,
+        file=sys.stdout,
+        by_groups=False,
+        by_targets=False,
+        by_categories=False,
+        by_users=False,
+        by_destinations=False,
+    ):
+        import csv
+
+        rows = self._rows(
+            by_groups=by_groups,
+            by_targets=by_targets,
+            by_categories=by_categories,
+            by_users=by_users,
+            by_destinations=by_destinations,
+        )
+
         fieldnames = list(rows[0].keys())
 
         writer = csv.DictWriter(file, fieldnames=fieldnames)
         writer.writeheader()
         writer.writerows(rows)
 
+    def xlsx(
+        self,
+        path="ecquote.xlsx",
+        by_groups=False,
+        by_targets=False,
+        by_categories=False,
+        by_users=False,
+        by_destinations=False,
+    ):
+        import xlsxwriter
+
+        rows = self._rows(
+            by_groups=by_groups,
+            by_targets=by_targets,
+            by_categories=by_categories,
+            by_users=by_users,
+            by_destinations=by_destinations,
+        )
+
+        fieldnames = list(rows[0].keys())
+        workbook = xlsxwriter.Workbook(path)
+        worksheet = workbook.add_worksheet()
+
+        def title(s):
+            s = s.replace("_", " ")
+            return s[0].upper() + s[1:]
+
+        header = workbook.add_format(
+            {
+                "bold": True,
+                "center_across": True,
+                "bottom": True,
+            }
+        )
+        for col, fieldname in enumerate(fieldnames):
+            worksheet.write(0, col, title(fieldname), header)
+
+        class StrFormat:
+            def __init__(self, name, workbook):
+                self.format = None
+                self.width = len(name)
+
+            def add(self, value):
+                self.width = max(self.width, len(value))
+
+        class IntFormat:
+            def __init__(self, name, workbook):
+                self.format = workbook.add_format()
+                self.format.set_num_format(3)
+                self.width = len(name)
+
+            def add(self, value):
+                self.width = max(self.width, (len(str(value)) + 2) // 3 * 4)
+
+        class BoolFormat:
+            def __init__(self, name, workbook):
+                self.format = None
+                self.width = len(name)
+
+            def add(self, value):
+                pass
+
+        class FloatFormat:
+            def __init__(self, name, workbook):
+                self.format = None
+                self.width = len(name)
+
+            def add(self, value):
+                pass
+
+        formats = {
+            str: StrFormat,
+            int: IntFormat,
+            bool: BoolFormat,
+            float: FloatFormat,
+        }
+
+        def cell_formats(colname, value):
+            if colname not in formats:
+                try:
+                    formats[colname] = formats[type(value)](colname, workbook)
+                except KeyError:
+                    print(colname, value, type(value))
+                    raise
+            formats[colname].add(value)
+            return formats[colname].format
+
+        for row, values in enumerate(rows):
+            for col, fieldname in enumerate(fieldnames):
+                value = values.get(fieldname)
+                worksheet.write(row + 1, col, value, cell_formats(fieldname, value))
+
+        for col, fieldname in enumerate(fieldnames):
+            # worksheet.write(0, col, fieldname)
+            worksheet.set_column(col, col, formats[fieldname].width * 0.9)
+
+        workbook.close()
+
 
 def costing(requests):
     return Costing(requests, CurrentCosting)
