@@ -290,6 +290,10 @@ class Request:
     def estimated_volume(self):
         return self.repres.estimated_volume()
 
+    @cached_method
+    def data_values(self):
+        return self.repres.data_values()
+
     @property
     @cached_method
     def repres(self):
@@ -577,3 +581,15 @@ class Request:
 
     def full_dump(self):
         return f"{self._groups} @ {self._attributes}"
+
+    # Used by freebie
+    @cached_method
+    def freebie_matching(self):
+        return (tuple((k, v) for k, v in sorted(self.postproc.items())), self.user)
+
+    @cached_method
+    def freebies(self):
+        freebies = []
+        for f in self.subset.free_data:
+            freebies.append(Request(self, f))
+        return freebies
